<?php
//  Synopsis:    Extract content metadata out of html-file-comments and write into xml-info-file
//  Author:      Dirk Henrici
//  Created:     2005-10-27
//  Last update: 2005-11-09
//  Comment:     Requires php5

  global $standard_frame;
  $standard_frame = 'Daten';

  function AddPersonNodes($persons, $currentnode, $nodename) {
    foreach ($persons as $person) {
      $newnode = $currentnode->ownerDocument->createElement($nodename);
      $currentnode->appendChild($newnode);
      foreach ($person as $personelementname=>$personelementvalue)
        $newnode->appendChild($currentnode->ownerDocument->createElement($personelementname, $personelementvalue));
    }
  }
  
  function WebPresenter_ExtractMetadata($file, &$doc) {
    global $standard_frame;

    echo "Metadaten-Extraktion: Beginn der Verarbeitung von Datei [$file]...\n";

    $success = true;
    $xpath = new DOMXPath($doc);
    $xpath->registerNamespace('icsy', $doc->documentElement->namespaceURI);
    $slides = $xpath->query('//icsy:lecture/topic/slide', $doc);
    foreach ($slides as $slidenode) {
      echo "  Bearbeite Folie [{$slidenode->getAttribute('id')}]...\n";
      //Quell-URL der Folie ermitteln
      $nodes = $xpath->query('media/link[1]/frame[@name="'. $standard_frame. '"]', $slidenode); //Index 0 gibt es nach W3C in xpath nicht
      if ($nodes->length==0)
        $nodes = $xpath->query('media/link[1]', $slidenode);
      if ($nodes->length==0) {
        echo "  Warnung: Folie [{$slidenode->getAttribute('id')}] hat keine URL angegeben.\n";
      } else {
        $media_url = $nodes->item(0)->getAttribute('url');

        //html-Dokument einlesen und Daten extrahieren
        $htmldoc = new DOMDocument();
        $htmlfile = file_get_contents($media_url);
        @$htmldoc->loadHTML($htmlfile);
        $content = array();
        $creation = array();
        $updates = array();
        $title = $htmldoc->getElementsByTagName('title')->item(0)->nodeValue;
        if (!empty($title)) $content['title'] = $title;
        $metatags = $htmldoc->getElementsByTagName('meta');
        foreach ($metatags as $metatag) {
          $meta_valid = false;
          $metaname = $metatag->getAttribute('name');
          if ($metaname=='description') {
            $content['description'] = $metatag->getAttribute('content');
            $meta_valid = true;
          }
          if ($metaname=='keywords') {
            $content['keywords'] = explode(',', $metatag->getAttribute('content'));
            for($i=0; $i<count($content['keywords']); $i++)
              $content['keywords'][$i] = trim($content['keywords'][$i]);
            $meta_valid = true;
          }
          if (substr($metaname, 0, 9) == 'creation:') {
            if ((substr($metaname, 9, 7) == 'author[') and (substr($metaname, 17, 2) == ']:')) { //Art und Weise hier beschrnkt auf neun Autoren
              if (!is_array($creation['authors'])) $creation['authors'] = array();
              $i = substr($metaname, 16, 1) - 1;
              if (!is_array($creation['authors'][$i])) $creation['authors'][$i] = array();
              if ((strlen($metaname)==23) and (substr($metaname, 19, 4) == 'name')) {
                $creation['authors'][$i]['name'] = $metatag->getAttribute('content');
                $meta_valid = true;
              }
              if ((strlen($metaname)==26) and (substr($metaname, 19, 7) == 'company')) {
                $creation['authors'][$i]['company'] = $metatag->getAttribute('content');
                $meta_valid = true;
              }
              if ((strlen($metaname)==24) and (substr($metaname, 19, 5) == 'email')) {
                $creation['authors'][$i]['email'] = $metatag->getAttribute('content');
                $meta_valid = true;
              }
            }
            if ((strlen($metaname)==13) and (substr($metaname, 9, 4) == 'date')) {
              $creation['date'] = $metatag->getAttribute('content');
              $meta_valid = true;
            }
            if ((strlen($metaname)==15) and (substr($metaname, 9, 6) == 'remark')) {
              $creation['remark'] = $metatag->getAttribute('content');
              $meta_valid = true;
            }
          }
          if ((substr($metaname, 0, 7) == 'update[') and (substr($metaname, 8, 2) == ']:')) { //Art und Weise hier beschrnkt auf neun Updates
            $update_no = substr($metaname, 7, 1) - 1;
            if ((substr($metaname, 10, 7) == 'editor[') and (substr($metaname, 18, 2) == ']:')) { //Art und Weise hier beschrnkt auf neun Editoren
              $i = substr($metaname, 17, 1) - 1;
              if (!array_key_exists($update_no, $updates)) {
                $updates[$update_no] = array();
                $updates[$update_no]['editors'] = array();
              }
              if (!is_array($updates[$update_no]['editors'][$i])) $updates[$update_no]['editors'][$i] = array();
              if ((strlen($metaname)==24) and (substr($metaname, 20, 4) == 'name')) {
                $updates[$update_no]['editors'][$i]['name'] = $metatag->getAttribute('content');
                $meta_valid = true;
              }
              if ((strlen($metaname)==27) and (substr($metaname, 20, 7) == 'company')) {
                $updates[$update_no]['editors'][$i]['company'] = $metatag->getAttribute('content');
                $meta_valid = true;
              }
              if ((strlen($metaname)==25) and (substr($metaname, 20, 5) == 'email')) {
                $updates[$update_no]['editors'][$i]['email'] = $metatag->getAttribute('content');
                $meta_valid = true;
              }
            }
            if ((strlen($metaname)==14) and (substr($metaname, 10, 4) == 'date')) {
              $updates[$update_no]['date'] = $metatag->getAttribute('content');
              $meta_valid = true;
            }
            if ((strlen($metaname)==16) and (substr($metaname, 10, 6) == 'remark')) {
              $updates[$update_no]['remark'] = $metatag->getAttribute('content');
              $meta_valid = true;
            }
          }
          if (!meta_valid)
            echo "  Warnung: Nicht erkanntes Meta-Tag [{$metaname}].\n";
        }
        
// *** print_r($content);
 
        //Metadaten in XML-Datei schreiben
        
        if (!empty($content) or !empty($creation) or !empty($updates)) {
          //metadatanode auf Metadata-Node der aktuellen Slide setzen, ggf. dazu Node anlegen
          $nodes = $xpath->query('metadata[1]', $slidenode); //Index 0 gibt es nach W3C in xpath nicht
          if ($nodes->length==0) {
            $metadatanode = $doc->createElement('metadata');
            $slidenode->insertBefore($metadatanode, $slidenode->firstChild);
          } else {
            // *** $currentnode = $slidenode->firstChild; // ginge so nur, wenn Reihenfolge der Elemente nicht gendert
            $metadatanode = $nodes->item(0);
          } 
          $warn_overwrite = false;
          //Wenn neue Metadaten vorliegen, alte lschen          
          $nodes = $xpath->query('content[1]', $metadatanode); //Index 0 gibt es nach W3C in xpath nicht
          if (!empty($content) and ($nodes->length>0)) {
            $warn_overwrite = true;
            $metadatanode->removeChild($nodes->item(0));
          }
          $nodes = $xpath->query('creation[1]', $metadatanode); //Index 0 gibt es nach W3C in xpath nicht
          if (!empty($creation) and ($nodes->length>0)) {
            $warn_overwrite = true;
            $metadatanode->removeChild($nodes->item(0));
// ***           $nodes->item(0)->parentNode->removeChild($nodes->item(0)); //allgemeine Variante
          }
          $nodes = $xpath->query('update[1]', $metadatanode); //Index 0 gibt es nach W3C in xpath nicht
          if (!empty($updates) and ($nodes->length>0)) {
            $warn_overwrite = true;
            $metadatanode->removeChild($nodes->item(0));
          }
          //neue Metadaten einfgen (rckwrts, damit gewohnte Reihenfolge einfacher einzuhalten ist)
          $currentnode = NULL;
          if (!empty($updates)) {
            $nodes = $xpath->query('praesentation[1]', $metadatanode); //Index 0 gibt es nach W3C in xpath nicht
            $currentnode = $nodes->item(0);
            for($i = count($updates) - 1; $i>=0; $i--) {
              $update = $updates[$i];
              $newnode = $doc->createElement('update');
              $metadatanode->insertBefore($newnode, $currentnode);
              AddPersonNodes($update['editors'], $newnode, 'editor');
              foreach ($update as $key=>$value)
                if ($key!='editors')
                  $newnode->appendChild($doc->createElement($key, $value));
              $currentnode = $newnode;
            }

          }
          if (!empty($creation)) {
            $newnode = $doc->createElement('creation');
            $metadatanode->insertBefore($newnode, $currentnode);
            foreach ($creation as $key=>$value)
              if ($key!='authors') {
                $newnode->appendChild($doc->createElement($key, $value));
              } else {
                AddPersonNodes($creation['authors'], $newnode, 'author');
              }
            $currentnode = $newnode;
          }
          if (!empty($content)) {
            $newnode = $doc->createElement('content');
            $metadatanode->insertBefore($newnode, $currentnode);
            foreach ($content as $key=>$value)
              if (($key!='keywords') and ($key!='remarks')) {
                $newnode->appendChild($doc->createElement($key, $value));
              } else {
                foreach ($value as $valuesub) {
                  if ($key=='keywords') $keysub = 'keyword';
                  if ($key=='remarks') $keysub = 'remark';
                  $newnode->appendChild($doc->createElement($keysub, $valuesub));
                }
              }
            $currentnode = $newnode;
          }
          //Aktuelle Folie ist fertig. Nacharbeiten durchfhren
          if ($warn_overwrite)
            echo "  Info: Metadaten in Slide [{$slidenode->getAttribute('id')}] wurden mit Metadaten aus [". basename($media_url). "] berschrieben.\n";
        } else {
          echo '  Info: Datei ['. basename($media_url). "] von Slide [{$slidenode->getAttribute('id')}] enthlt keine verwertbaren Metadaten.\n";
        }
      }
    }

//    $doc->save($file);
// *** echo $doc->saveXML();

    if ($success) echo "Metadaten-Extraktion: Verarbeitung der Datei [$file] ohne Fehler abgeschlossen.\n";
    return $success;
  } 
?>
