package mms;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.jdom.xpath.XPath;

public class SVGDistributor {
	
	private File rootDir;
	private String separator;
	
	public static void main(String[] args) {
		try {
			new SVGDistributor().generate(args);
			System.out.println("program finished!");
		} catch (IOException e) {
			System.out.println("IOException occured!");
			e.printStackTrace();
		}
	}
	
	public SVGDistributor() {
		separator = File.separator;
	}
	
	// TODO: aus Datei einlesen
	private void printHelp() {
		System.out.println("Informationen zum ICSY WebPresenter SVG Distributor! " +
				"\n Das Java-Programm verwendet als Eingabe SVG/SVGZ-Dateien und " +
				"generiert aus einem HTML-template (template.html) eine vollstndige " +
				"HTML-Seite. \n Diese Seite ist nur zur Weiterleitung gedacht. Sie " +
				"prft, ob der Browser das Adobe SVG Plugin installiert hat. \n" +
				"Falls " +
				"ja wird geprft, ob der Client eine gleiche oder hhere " +
				"Auflsung eingestellt hat, als die Vorlesung bentigt. \n Falls der " +
				"Client eine kleinere Auflsung hat, wird die skalierbare Version " +
				"aufgerufen (<svg-dateiname>_scaled.svg(z))." +
				"\n Falls der Client nicht das Adobe Plugin installiert hat, wird " +
				"an eine Informationsseite weitergeleitet (SVG_info.html)." +
				"\n Als Eingabe knnen einzelne Dateien oder auch verschachtelte " +
				"Ordner angegeben werden: Es muss '-i:' oder '-d:' spezifiziert " +
				"werden:\n" +
				"'-i:<svg-file>': spezifiziert eine SVG-Eingabedatei (1.Argument))\n" +
				"'-d:<Verzeichnis>': spezifiziert ein Verzeichnis (1.Argument)\n" +
				"'-o:<Zielverzeichnis>': spezifiziert ein Zielverzeichnis " +
				"(optional, falls vorhanden, dann 2. Argument, default ist " +
				"'<input-Verzeichnis>/svg').\n" +
				"'-svg' bzw. '-svgz': Welche Dateien sollen Verarbeitet werden. Nur" +
				" in Verbindung mit '-d:<Pfad>', optional, default ist '-svgz'.\n" +
				"'-nocopy_resources' bzw. '-copy_resources': Sollen die Resourcen " +
				"ebenfalls ins Zielverzeichnis kopiert werden (Bilder, Audio, " +
				"svg(z), xml)? Optional, default ist '-nocopy_resources'.");
		System.exit(0);
		
	}
	
	private void generate(String[] args) throws IOException {
		// print help
		if (args.length == 1) {
			if (args[0].equals("--help")) printHelp();
		}
		// else...
		boolean svgFlag = false;
		boolean svgzFlag = false;
		boolean iFlag = false;
		boolean dFlag = false;
		boolean oFlag = false;
		boolean nocopyFlag = false;
		boolean copyFlag = false;
		
		int size = args.length;
		int i;
		String arg;
		// every flag can be set at most once
		for (i = 0; i < size; i++) {
			arg = args[i];
			if (arg.equals("-svg")) {
				if (svgFlag) printInfo();
				svgFlag = true;
			}
			else if (arg.equals("-svgz")) {
				if (svgzFlag) printInfo();
				svgzFlag = true;
			}
			// if specified then it must be the first arg
			else if (arg.startsWith("-i:")) {
				if (iFlag) printInfo();
				if (i > 0) printInfo();
				iFlag = true;
			}
			// if specified then it must be the first arg
			else if (arg.startsWith("-d:")) {
				if (dFlag) printInfo();
				if (i > 0) printInfo();
				dFlag = true;
			}
			// if specified then ist must be the 2nd arg
			else if (arg.startsWith("-o:")) {
				if (oFlag) printInfo();
				if (i != 1) printInfo();
				oFlag = true;
			}
			else if (arg.equals("-nocpy_resources")) {
				if (nocopyFlag) printInfo();
				nocopyFlag = true;
			}
			else if (arg.equals("-copy_resources")) {
				if (copyFlag) printInfo();
				copyFlag = true;
			}
		}
		// svgFlag bzw svgzFlag must not be specified if iFlag = true
		if (svgFlag & svgzFlag) {
			if (iFlag) printInfo();
		}
		// check if argument setting is valid
		// only one of both flags can be true, or both false
		if (svgFlag & svgzFlag) printInfo();
		// default value: svgzFlag = true
		if (!svgFlag & !svgzFlag) svgzFlag = true;
		// only one of -i or -d can be specified
		if (iFlag & dFlag) printInfo();
		// -i or -d must be set
		if (!iFlag & !dFlag) printInfo();
		// only one of -nocopy_resoures or -copy_resources can be set
		if (copyFlag & nocopyFlag) printInfo();
		boolean copyResources, isExtensionSvg;
		if (copyFlag) copyResources = true;
		else copyResources = false;
		if (svgFlag) isExtensionSvg = true;
		else isExtensionSvg = false;
		String target;
		// extract -o:
		if (oFlag) target = args[1].substring(3,args[1].length());
		else target = null;
		// proceed input file
		if (iFlag) {
			// extract -i:
			String file = args[0].substring(3, args[0].length());
			proceed(new File(file), new File(target), copyResources);
		}
		// proceed input folder
		else if (dFlag) {
			// extract -d:
			String inputPath = args[0].substring(3,args[0].length());
			rootDir = new File(inputPath);
			if (isExtensionSvg) {
				proceedDir(rootDir, "svg", target, copyResources);
			}
			else proceedDir(rootDir, "svgz", target, copyResources);
		}		
	}
	
	private void printInfo() {
		System.err.println("Rufen Sie das Programm mit dem Argument '--help' auf, " +
				"um Informationen zur Benutzung zu bekommen.\n");
		System.exit(0);
	}
	
	private void proceedDir(File inputDir, String fileExtension, 
			String targetFolderRoot, boolean copyResources) throws IOException {
		// check condition whether argument for input dir specifies an existing
		// folder
		if ((inputDir == null) | (!inputDir.isDirectory())) {
			System.err.println("Error: folder '" + inputDir.getCanonicalPath() 
					+ "' does not exist!\n");
			return;
		}
		// check whether input dir is target root dir. if true
		// skip processing
		else {
			if (targetFolderRoot != null) {
				if (inputDir.getCanonicalPath().equals(targetFolderRoot)) return;
			}
		}
		// create target folder
		String relativeDir, targetFolder;
		// <rootDir>/svg/<inputDir>
		if (targetFolderRoot == null) {
			relativeDir = getRelativeDir(inputDir.getCanonicalPath());
			targetFolderRoot = rootDir.getCanonicalPath() + separator + "svg";
			targetFolder = targetFolderRoot + relativeDir;
		}
		// <target dir>/<input dir>
		else {
			relativeDir = getRelativeDir(inputDir.getCanonicalPath());
			targetFolder = targetFolderRoot + relativeDir;
		}
		File target = new File(targetFolder);
		if (!target.isDirectory()) {
			target.mkdir();
		}
		String[] fileNames = inputDir.list();
		int size = fileNames.length;
		int i; 
		File svg;
		String name, path;
		// proceed all input files in dir
		for (i = 0; i < size; i++) {
			path = inputDir.getCanonicalPath();
			name = fileNames[i];
			svg = new File(path, name);
			// check if input is a file or dir
			// recursive call
			if (svg.isDirectory()) {
				proceedDir(svg, fileExtension, targetFolderRoot, copyResources);
			}
			else {
				if (!name.endsWith("." + fileExtension)) {
//					System.err.println("Skip file: '" + name + "'!");
					continue;
				}
				String svgFilename = svg.getCanonicalPath();
				// filter only svg or svgz files (without the post fix '_scaled')
				if (!svgFilename.endsWith("_scale." + fileExtension)) {
					proceed(svg, target, copyResources);
				}
			}
		}
	}
	
	private String getRelativeDir(String dir) throws IOException {
		String newDir = dir.replaceFirst(rootDir.getCanonicalPath(), "");
		return newDir;
	}
	
	private void proceed(File svg, /*File xml,*/ File folder, boolean copyResources) {
		Resolution[] resolutions = null;
		File xml;
		try {
			System.out.println("Proceed svg file: " + svg.getCanonicalPath()); 
			String fileName = svg.getCanonicalPath();
			fileName = fileName.substring(0, fileName.lastIndexOf("."));
			fileName += ".xml";
			xml = new File(fileName); 
			resolutions = checkInput(xml.getCanonicalPath());
		} catch (Exception e) {
			System.err.println("An error occured while processing corresponding " +
					"xml file. Skip processing of svg " +
					"file: " + svg.getName() + "\n");
			return;
		} 
		File output = null;
		try {
			output = build(svg, resolutions);
		} catch (Exception e) {
			System.err.println("An error occured while generating HTML output " +
					"file! Skip processing of svg file: " + svg.getName() + "\n");
			return;
		}
		try {
			copyFilesToFolder(svg, xml, output, folder, copyResources);
		} catch (Exception e) {
			System.err.println("An error occured while copying files to target " +
					"folder.\n");
		}
	}
	
	private void copyFilesToFolder(File svg, File xml, File html, 
			File folder, boolean copyResources) throws Exception {
		if (folder == null) {
			folder = new File(svg.getParent() + separator + "svg");
			folder.mkdir();
		}
		if (!folder.isDirectory()) {
			// folder does not exist
			// create folder
			boolean ok = folder.mkdir();
			if (!ok) {
				throw new Exception();
			}
		}
		// copy files
		// copy resource files only if flag is true
		if (copyResources) {
			copy(svg, folder);
			// copy other svg documents
			String svgScaled = svg.getCanonicalPath();
			svgScaled = svgScaled.substring(0, svgScaled.lastIndexOf("."));
			svgScaled += "_scaled.svg";
			if (svg.getName().endsWith("svgz")) svgScaled += "z";
			copy(new File(svgScaled), folder);
			copyResources(xml, folder);
			copy(xml, folder);
		}
		// copy generated HTML file
		copy(html, folder);
		// also copy SVG_info.html
		copy(new File("mms" + separator + "SVG_info.html"), folder);
	}
	
	private void copy(File file, File targetFolder) throws Exception {
		// check whether file exists
		if (!file.isFile()) {
			// file does not exist
			return;
		}
		// create new file in target folder
		String folderName = targetFolder.getCanonicalPath();
		String fileName = file.getName();
		File copy = new File(folderName + separator + fileName);
		System.out.println("copy " + copy.getName() + " to " + folderName 
				+ separator);
		FileInputStream in = new FileInputStream(file); 
        FileOutputStream out = new FileOutputStream(copy); 
        int c; 
        while ((c = in.read()) != -1) {
           out.write(c); 
        }
        in.close(); 
        out.close(); 
	}
	
	private void copyResources(File xml, File folder) throws Exception {
		// retrieve images and audio files from xml file
		String xmlPath = xml.getParent();
		XPath x;
		List results;
		x = XPath.newInstance("//slide/media");
		results = x.selectNodes(getDoc(xml));
		// no slides available -> no resources to copy
		if (results == null) return;
		Element current;
		String audioFile, snapshotFile, pictureFile;
		int size = results.size();
		int i;
		for (i = 0; i < size; i++) {
			current = (Element) results.get(i);
			audioFile = current.getChild("audio").getAttributeValue("filename");
			snapshotFile = 
				current.getChild("snapshot").getAttributeValue("filename");
			pictureFile = current.getChild("picture").getAttributeValue("filename");
			copyResource(xmlPath + separator + audioFile, folder);
			copyResource(xmlPath + separator + pictureFile, folder);
			copyResource(xmlPath + separator + snapshotFile, folder);
		}
	}
	
	private void copyResource(String fileName, File folder) throws Exception {
		File file = new File(fileName);
		if (!file.isFile()) {
			// specified file not available -> nothing to copy
			System.err.println("File " + fileName + " not found!\n");
		}
		else {
			copy(file, folder);
		}
	}
	
	private Document getDoc(File input) throws Exception {
		// check if schema file is located in root dir
		File schema = new File("icsy.xsd");
		if (!schema.exists()) {
			System.err.println("Error: xml schema file 'icsy.xsd' not found! " +
					"Make sure it is located in application's root directory. " +
					"Program terminated!");
			System.exit(0);
		}
		SAXBuilder builder = 
            new SAXBuilder("org.apache.xerces.parsers.SAXParser", true);
        builder.setFeature("http://apache.org/xml/features/validation/schema", true);        
        builder.setProperty("http://apache.org/xml/properties/schema/" +
        		"external-schemaLocation", 
        		"http://www.icsy.de" +
        		" icsy.xsd");
        return builder.build(new FileInputStream(input)); 
	}
	
	private Resolution[] checkInput(String xmlFile) throws Exception {
		Resolution[] resolutions = null;
		Document inputDoc = getDoc(new File(xmlFile));
        List results = null;
			XPath x = XPath.newInstance("//slide");
			results = x.selectNodes(inputDoc);
			resolutions = getResolutions(results);
        return resolutions;
	}
	
	private Resolution[] getResolutions(List slides) {
		Resolution[] resolutions = null;
		if (slides != null) {
			int size = slides.size();
			resolutions = new Resolution[size];
			int i;
			Resolution resolution;
			Element current;
			int width, height;
			for (i = 0; i < size; i++) {
				current = (Element) slides.get(i);
				width = Integer.parseInt(
						current.getAttribute("canvaswidth").getValue());
				height = Integer.parseInt(
						current.getAttribute("canvasheight").getValue());
				resolution = new Resolution();
				resolution.setWidth(width);
				resolution.setHeight(height);
				resolutions[i] = resolution;
			}
		}
		return resolutions;
	}
	
	private File build(File svgFile, Resolution[] resolutions) throws Exception {
		SAXBuilder builder = 
			new SAXBuilder("org.apache.xerces.parsers.SAXParser", false);
		File templateFile = new File("mms" + separator + "template.html");
			Document template = builder.build(templateFile);
			// add javascript code to template document. Needed infos are extracted 
			// from svg file and Resolution-Array
			addJS(template, svgFile, resolutions);
			String svgFileName = svgFile.getCanonicalPath();
			// construct valid HTML file from template document
			String outputName = 
				svgFileName.substring(0, svgFileName.lastIndexOf("."));
			outputName += ".html";
			File output = new File(outputName);
			writeFile(template, output);
			// return reference to generated HTML file
			return output;
	}
	
	private String createArray(Resolution[] resolutions) {
		String js = "var resolutions = new Array(";
		int size = resolutions.length;
		int i, width, height;
		Resolution resolution;
		for (i = 0; i < size; i++) {
			resolution = resolutions[i];
			width = resolution.getWidth();
			height = resolution.getHeight();
			js += "new Array(" + width + "," + height + ")";
			if (i != (size - 1)) js += ", ";
			else js += ");";
		}
		return js;
	}
	
	private void addJS(Document doc, File svg, Resolution[] resolutions) 
		throws JDOMException {
		String link = svg.getName();
		XPath x;
		List results;
		x = XPath.newInstance("//script[@id='resolution']");
		results = x.selectNodes(doc);
		Element script = (Element) results.get(0);
		// add content to element
		script.addContent(createArray(resolutions));
		x = XPath.newInstance("//body/@onload");
		results = x.selectNodes(doc);
		if (results != null) {
			// link2 is the link to the scalable SVG document
			String link2 = link.substring(0, link.lastIndexOf("."));
			link2 += "_scale.svg";
			if (link.endsWith("svgz")) {
				link2 += "z";
			}
			Attribute onload = (Attribute) results.get(0);
			onload.setValue("if (hasSVGSupport == true) { " +
					"if (needForScale == true) { " +
					"window.location.href='" + link2 + "'} " +
					"else { window.location.href='" + link + "'}"	+
				"} " +
				"else {window.location.href=infoPage}");
		}
	}
	
	private void writeFile(Document doc, File outputFile) {
		Format format = Format.getPrettyFormat();
	    format.setEncoding("iso-8859-1");
	    format.setIndent("\t");
	    XMLOutputter outputter = new XMLOutputter(format);
        try { 
            outputter.output(doc, new FileOutputStream(outputFile));
        } catch (Exception e) {
        		System.err.print("Error: Couldn't write " + outputFile.getName() + "\n");
        }
	}
}

class Resolution {
	private int width;
	private int height;
	public int getHeight() {
		return height;
	}
	public void setHeight(int height) {
		this.height = height;
	}
	public int getWidth() {
		return width;
	}
	public void setWidth(int width) {
		this.width = width;
	}
}
