


/***************************************************
*                                                  *
*   TextWrap || <flow:text>                        *
*                                                  *
*   A class for naive text-wrapping with SVG 1.0   *
*                                                  *
***************************************************/

/**
 *
 * SVG
 *
 **/
 
function SVG () {
}


/*****
 *
 * init svg document
 * create text wrapping
 * init context menu
 * manipulate DOM-tree at runtime
 *
 *****/

var SVGDocument;
var targetNode = null;

function init(evt) {
	SVGDocument = evt.target.ownerDocument;
	var newMenuRoot = parseXML(printNode(document.getElementById('myCustomMenu')), contextMenu);
    contextMenu.replaceChild( newMenuRoot, contextMenu.firstChild );
	TextWrap._init();
}

SVG.ns = 'http://www.w3.org/2000/svg';

/**
 *
 * TextWrap
 *
 **/
 
/**
*
*
* configuration parameters
*
*/ 
var color = '#333399';
var lines_overall = 0;
var width = 410;
var x = 10;
var y_initial = 70;
var y = y_initial;
var size = 12;
var intervall = 16;
var height_image = 155;

function TextWrap (id, node) {
	this._id = id;
	this._node = node;
	this._string = null;
	this._x = null;
	this._y = null;
	this._width = null;
	this._font = null;
	this._size = null;
	this._align = null;
	this._quality = null;
	this._interval = null;
	this._svg = null;
	this._lines = null;
	this._initialized = false;
	this._construct();
}

TextWrap.ns = 'http://xmlns.graougraou.com/svg/text/';

TextWrap._instances = new Array();

var panelCount = 0;
var k = 1;


TextWrap._init = function () {
	var elements = document.documentElement.getElementsByTagNameNS(this.ns, 'wrap');
	for (var i=0; i<elements.length; i++) {
		this._instances.push( new TextWrap(i, elements.item(i)) );
		// update set elements' attributes 
		if ((k % 4) == 0) {
			SVGDocument = evt.target.ownerDocument;
			id = 'clip_bottom'+panelCount;
   			SVGDocument.getElementById(id).setAttribute("to", 55 + lines_overall * intervall + 45 + height_image);
   			id = 'clip_right'+panelCount;
   			SVGDocument.getElementById(id).setAttribute("to", lines_overall * intervall - 5 + 45 + height_image );
   			id = 'progresspanel'+panelCount;
   			SVGDocument.getElementById(id).setAttribute("to", 45 + height_image + lines_overall * intervall);
   			id = 'dropshaddow'+panelCount;
   			SVGDocument.getElementById(id).setAttribute("to", 45 + height_image + lines_overall * intervall);
   			id = 'snapshot_x'+panelCount;
   			SVGDocument.getElementById(id).setAttribute("to", "10");
   			id = 'snapshot_y'+panelCount;
   			SVGDocument.getElementById(id).setAttribute("to", 55 + 30 + lines_overall * intervall);
   			panelCount++;
   			y = y_initial;
   			lines_overall = 0;
   		}
   		k++;
	}
}

/**
 *
 * Processing methods
 *
 **/

TextWrap.prototype._construct = function () {

	this._build();
	this._svg.setAttribute('style', this._node.getAttribute('style'));
	var style = this._svg.style;
	this._node.normalize();
	this.setString( this._node.firstChild.data );
	this.setX( x );
	this.setY( y );
	this.setWidth( width );
	this.setTextAlign( style.getPropertyValue('text-align') );
	this.setFontFamily( style.getPropertyValue('font-family') );
	this.setFontSize( size );
	this.setTextRendering( style.getPropertyValue('text-rendering') );
	this.setLineInterval( intervall );
	this._splitString();
	this._layout();
	this._initialized = true;
}

TextWrap.prototype._build = function () {
	var element = document.createElementNS(SVG.ns, 'text');
	element.setAttribute('fill', color);
	var node = this._node;
	var nextElement = null;
	while (node.nextSibling) {
		if (node.nextSibling.nodeType == 1) {
			nextElement = node.nextSibling;
			break;
		} else {
			node = node.nextSibling;
		}
	}
	if (nextElement) {
		var test = this._node.parentNode.insertBefore(element, nextElement);
	} else {
		this._node.parentNode.appendChild(element);
	}
	element.appendChild(document.createTextNode(''));
	this._svg = element;
}

TextWrap.prototype._splitString = function () {
	this._hide();
	this._clear();
	var words = this._string.split(' ');
	var lines = new Array();
	var line = new Array();
	var length = 0;
	var prevLength = 0;	
	while (words.length) {
		var word = words[0];
		this._svg.firstChild.data = line.join(' ') + ' ' + word;
		length = this._svg.getComputedTextLength();
		if (length > this._width) {
			if (!words.length) {
				line.push(words[0]);
			}
			lines.push( new Line(prevLength, line) );
			line = new Array();
		} else {
			line.push(words.shift());
		}
		prevLength = length;
		if (words.length == 0) {
			lines.push( new Line(0, line) );
		}
	}
	this._lines = lines;
	y += lines.length * parseInt(this._interval) + parseInt(this._interval);
	// + 1 for an extra line break
	lines_overall += lines.length + 1;
}

TextWrap.prototype._layout = function () {
	this._clear();
	var lines = (new Array(0)).concat(this._lines);
	var anchor = 'start';
	if (this._align == 'center') {
		anchor = 'middle';	
	} else if (this._align == 'right') {
		anchor = 'end';
	}
	for (var i=0; i<lines.length; i++) {
		var x = 0;		
		line = lines[i]; 
		this._svg.appendChild( document.createTextNode(' ') );
		var tspan = document.createElementNS(SVG.ns, 'tspan');
		tspan.appendChild( document.createTextNode(line._words.join(' ')) ); 
		if (this._align == 'justify') {
			var space = (this._width - line._width) / (line._words.length - 1);
			space = (i != lines.length - 1) ? space : 0;
			tspan.style.setProperty('word-spacing', space + 'px');
		} else if (this._align == 'center') {
			anchor = 'middle';
			x = this._width / 2;
		} else if (this._align == 'right') {
			anchor = 'end';
			x = this._width;
		}
		tspan.setAttribute('x', x);
		tspan.setAttribute('dy', i ? this._interval : '1em');
		this._svg.appendChild(tspan);
	}	
	this._svg.style.setProperty('text-anchor', anchor);
	this._show();
}

/**
 *
 * Utility methods
 *
 **/

TextWrap.prototype._hide = function () {
	this._svg.style.setProperty('opacity', '0');
}

TextWrap.prototype._show = function () {
	this._svg.style.setProperty('opacity', '1');
}

TextWrap.prototype._clear = function () {
	while (this._svg.hasChildNodes()) {
		this._svg.removeChild(this._svg.firstChild);
	}
	this._svg.appendChild(document.createTextNode(''));
}

/**
 *
 * GET / SET
 *
 * Getters and setters for CSS properties and XML attributes,
 * will be updated when mutation events will be implemented
 *
 **/

TextWrap.prototype.getX = function () {
	return this._x;
}

TextWrap.prototype.setX = function (x) {
	if (x != this._x) {
		this._x = x;
		this._svg.setAttribute('transform', 'translate(' + this._x + ' ' + this._y + ')');
	} 
}

TextWrap.prototype.getY = function () {
	return this._y;
}

TextWrap.prototype.setY = function (y) {
	if (y != this._y) {
		this._y = y;
		this._svg.setAttribute('transform', 'translate(' + this._x + ' ' + this._y + ')');
	} 
}

TextWrap.prototype.getWidth = function () {
	return this._width;
}

TextWrap.prototype.setWidth = function (width) {
	if (width != this._width) {
		this._width = width;
		if (this._initialized) {
			this._splitString();
			this._layout();
		}
	} 
}

TextWrap.prototype.getTextAlign = function () {
	return this._align;
}

TextWrap.prototype.setTextAlign = function (align) {
	if (align != this._align) {
		this._align = align;
		if (this._initialized) {
			this._layout();
		}
	} 
}

TextWrap.prototype.getString = function () {
	return this._string;
}

TextWrap.prototype.setString = function (string) {
	if (string != this._string) {
		this._string = string;
		if (this._initialized) {
			this._splitString();
			this._layout();
		}
	} 
}

TextWrap.prototype.getFontFamily = function () {
	return this._font;
}



TextWrap.prototype.setFontFamily = function (font) {
	if (font != this._font) {
		this._font = font;
		this._svg.style.setProperty('font-family', this._font);
		if (this._initialized) {
			this._splitString();
			this._layout();
		}
	} 
}

TextWrap.prototype.getFontSize = function () {
	return this._size;
}

TextWrap.prototype.setFontSize = function (size) {
	if (size != this._size) {
		this._size = size;
		this._svg.style.setProperty('font-size', this._size);
		if (this._initialized) {
			this._splitString();
			this._layout();
		}
	} 
}

TextWrap.prototype.getTextRendering = function () {
	return this._quality;
}

TextWrap.prototype.setTextRendering = function (quality) {
	if (quality != this._quality) {
		this._quality = quality;
		this._svg.style.setProperty('text-rendering', this._quality);
		if (this._initialized) {
			this._splitString();
			this._layout();
		}
	} 
}

TextWrap.prototype.getLineInterval = function () {
	return this._interval;
}

TextWrap.prototype.setLineInterval = function (interval) {
	if (interval != this._interval) {
		this._interval = interval;
		if (this._initialized) {
			var element = this._svg.firstChild;
			var count = 0;
			while (element) {
				if (element.nodeName == 'tspan') {
					if (count) {
						element.setAttribute('dy', this._interval);
					}
					count++;
					if (count == this._lines.length) {
						break;
					}
				}
				element = element.nextSibling;
			}
		}
	}
}

/*****
 *
 * Line
 *
 *****/
 
function Line (width, words) {
	this._width = width;
	this._words = words;
}

/*****
 *
 * add animation elements to svg document
 * at runtime after start button was clicked
 *
 *****/

var button_id = 'additional_info_button1';

infoButtonAlterID = function (evt) {
	
	var button = SVGDocument.getElementById(button_id);
	if (button_id == 'additional_info_button1') {
		button_id = 'additional_info_button2';
	}
	else button_id = 'additional_info_button1';
	button.setAttribute('id', button_id);
}

var progressbarInit = false;

addElementsToProgressbar = function (evt) {
	var progress = SVGDocument.getElementById('progress');
	var animate_hide = SVGDocument.createElement('animate');
	animate_hide.setAttribute('attributeName', 'opacity');
	animate_hide.setAttribute('begin', 'mouseout');
	animate_hide.setAttribute('to', '0.2');
	animate_hide.setAttribute('dur', '1s');
	animate_hide.setAttribute('fill', 'freeze');
	progress.appendChild(animate_hide);
	var animate_hide2 = SVGDocument.createElement('animate');
	animate_hide2.setAttribute('attributeName', 'opacity');
	animate_hide2.setAttribute('begin', 'mousedown');
	animate_hide2.setAttribute('dur', '1s');
	animate_hide2.setAttribute('to', '0');
	animate_hide2.setAttribute('fill', 'freeze');
	progress.appendChild(animate_hide2);
	var animate_hide3 = SVGDocument.createElement('animate');
	animate_hide3.setAttribute('attributeName', 'opacity');
	animate_hide3.setAttribute('begin', 'mouseup');
	animate_hide3.setAttribute('dur', '1s');
	animate_hide3.setAttribute('to', '0.2');
	animate_hide3.setAttribute('fill', 'freeze');
	progress.appendChild(animate_hide3);
	progressbarInit = true;
}

/*****
 *
 * init, pause and continue timer
 *
 *****/

var active = null;
var duration;
var millis;
var millis2;

setupTimer = function (end) {
	duration = end;
	millis = 0;
	temp = duration % 1000;
	duration -= temp;
	millis2 = duration;
	//temp = millis2 % 1000;
	//millis2 -= temp;
	active = window.setInterval('updateTimer()', 1000); 
	
}

updateTimer = function () {
	if (millis == duration) {
		 window.clearInterval(active);
		 return;
	}
	millis += 1000;
	millis2 -= 1000;
  	setTimer(millis, "timer");
  	setTimer(millis2, "timer2");
}

function setTimer(ms, id) {
	var currentTime = new Date(ms);
	sHours = Math.floor( currentTime / 3600000 );
	sMin = currentTime.getMinutes(); 
  	sSec = currentTime.getSeconds(); 
  	sMin = ( sMin < 10 ) ? "0" + sMin : sMin;
  	sSec = ( sSec < 10 ) ? "0" + sSec : sSec;
  	value = sHours + ":" + sMin + ":" + sSec;
  	if (id == "timer2") value = "-" + value;
  	timer = SVGDocument.getElementById(id);
	timer.getFirstChild().setNodeValue(value);
}

var nrTimer = 0;

/*****
 *
 * switch between both timer
 *
 *****/

function switchTimer() {
	nrTimer++;
	if (nrTimer % 2 == 0) nrTimer = 0;
	if (nrTimer == 0) {
		SVGDocument.getElementById("timer").setAttribute("opacity", "1");
		SVGDocument.getElementById("timer2").setAttribute("opacity", "0");
	}
	else {
		SVGDocument.getElementById("timer2").setAttribute("opacity", "1");
		SVGDocument.getElementById("timer").setAttribute("opacity", "0");

	}
}

/****
*
* pause / unpause animations and timer
*
*****/

var play = false;

function pause() {
	if (play == false) play = true;
	else play = false;
	if (play) {
		pauseStart = new Date();
		//SVGRoot.pauseAnimations();
		if (active != null) 
		{
			window.clearInterval(active);
		}
	}
	else {
		if (active != null) active = window.setInterval('updateTimer()', 1000); 
	}
}

/***
*
* hide / unhide information panel and timer
*
***/

var panelHidden = false;

function hideInfoPanel() {
	
	if (panelHidden) panelHidden = false;
	else panelHidden = true;
	if (panelHidden) {
		if (progressbarInit) {
			SVGDocument.getElementById("info-panel").setAttribute("display", "none");
			toggleCheckmark("item_infoPanel", "no");
		}
	}
	else {
		if (progressbarInit) {
			SVGDocument.getElementById("info-panel").setAttribute("display", "block");
			toggleCheckmark("item_infoPanel", "yes");
		}
	}
}

var timerHidden = false;

function hideTimer() {
	if (timerHidden) timerHidden = false;
	else timerHidden = true;
	if (timerHidden) {
		if (progressbarInit) {
			SVGDocument.getElementById("timer_g").setAttribute("display", "none");
			toggleCheckmark("item_timer", "no");
		}
	}
	else {
		if (progressbarInit) {
			SVGDocument.getElementById("timer_g").setAttribute("display", "block");
			toggleCheckmark("item_timer", "yes");
		}
	}
}

/*****
 *
 * init and update context menu
 *
 *****/

function toggleCheckmark(id, value) {
	var menuItems = contextMenu.childNodes.item(0).childNodes;
      
      //step through all nodes and get reference to item wanted
      for (i = 0; i < menuItems.length; i++) {
         if (1 == menuItems.item(i).nodeType) {
            current = menuItems.item(i);
            if (current.getAttribute("id") == id) {
            	current.setAttribute("checked", value);
            }
         }
     }
}

function updateContextMenu() {
	var menuItems = contextMenu.childNodes.item(0).childNodes;
      
      //step through all nodes and get reference to item wanted
      for (i = 0; i < menuItems.length; i++) {
         if (1 == menuItems.item(i).nodeType) {
            current = menuItems.item(i);
            if ((current.getAttribute("id") == "item_infoPanel") || 
            	(current.getAttribute("id") == "item_timer")) {
            	current.setAttribute("enabled", "yes");
            }
         }
     }
}