package eLecture;

import java.util.List;

import org.jdom.Element;

/**
 * @author seb
 * 
 * wrap informations of a metadata XML element of the input file and provide access
 * methods. Metadata can be available for lecture, topic and slide. If no metadata is 
 * available then set every field to null.
 * 
 */
public class Metadata {
	private String title;
	private String description;
	private Person[] authors;
	private Person[] editors;
	private Person speaker;
	private String site;
	private String date;
	
	/**
	 * 
	 * construct object from XML metadata element
	 * 
	 * @param metadata XML element
	 */
	public Metadata(Element meta1, Element meta2, Element meta3) {
		// retrieve title
		title = getMetainfo("content", "title", meta1, 
				meta2, meta3);
		// retrieve description
		description = getMetainfo("content", "description", meta1, 
				meta2, meta3);
		// retrieve authors
		authors = getPersonMetainfos("creation", "author", meta1, meta2, 
				meta3);
		// retrieve editors
		editors = getPersonMetainfos("update", "editor", meta1, meta2, 
				meta3);
		speaker = getPersonMetainfo("presentation", "speaker", meta1, meta2, 
				meta3);
		site = getMetainfo("presentation", "site", meta1, meta2, meta3);
		date = getMetainfo("presentation", "date", meta1, meta2, meta3);
	}
	
	private Person getPersonMetainfo(String category, String metainfo, 
			Element lectureMeta, Element topicMeta, Element slideMeta) {
		Element[] e = retrieveMetainfos(category, metainfo, lectureMeta, topicMeta, 
				slideMeta);
		return getPersonMetainfoHelper(e);
	}
	
	private Person[] getPersonMetainfos(String category, String metainfo, 
			Element lectureMeta, Element topicMeta, Element slideMeta) {
		Element[] e = retrieveMetainfos(category, metainfo, lectureMeta, topicMeta, 
				slideMeta);
		return getPersonMetainfosHelper(e);
	}
	
	private String getMetainfo(String category, String metainfo, 
			Element meta1, Element meta2, Element meta3) {
		Element[] e = retrieveMetainfos(category, metainfo, meta1, meta2, meta3);
		if (e != null) {
			if (e.length != 0) {
				return e[0].getTextTrim();
			}
		}
		return null;
	}
	
	private Element[] retrieveMetainfos(String category, String metainfo, 
			Element meta1, Element meta2, Element meta3) {
		// look for metainfos in lecture's subtree 
		Element[] e = retrieveMetainfosHelper(category, metainfo, meta1);
		// metainfo found in lecture element's subtree
		if (e != null) return e;
		// look for metainfos in topic's subtree
		e = retrieveMetainfosHelper(category, metainfo, meta2);
		// metainfo found in topic element's subtree
		if (e != null) return e;
		// look for metainfos in slide's subtree
		e = retrieveMetainfosHelper(category, metainfo, meta3);
		if (e != null) return e;
		// no metainfos found
		return null;
	}
	
	private Element[] retrieveMetainfosHelper(String category, String metainfo, 
			Element metadata) {
		if (metadata == null) return null;
		Element[] e = null;
		Element categoryElement, current;
		List result;
		if (metadata != null) {
			categoryElement = metadata.getChild(category);
			if (categoryElement != null) {
				result = categoryElement.getChildren(metainfo);
				// metainfo(s) found in subtree of metadata
				// create an array of elements
				if (result != null) {
					int i;
					int size = result.size();
					e = new Element[size];
					for (i = 0; i < size; i++) {
						current = (Element) result.get(i);
						e[i] = current;
					}
				}
			}
		}
		return e;
	}
	
	private Person getPersonMetainfoHelper(Element[] metainfo) {
		if (metainfo == null) return null;
		Person[] persons = getPersonMetainfosHelper(metainfo);
		if (persons == null) return null;
		return persons[0];
	}
	
	
	private Person[] getPersonMetainfosHelper(Element[] metainfos) {
		if (metainfos == null) return null;
		Person[] persons = null;
		int size = metainfos.length;
		persons = new Person[size];
		Person person;
		String name;
		String company;
		String[] email;
		Element current;
		int i;
		for (i = 0; i < size; i++) {
			current = metainfos[i];
			name = getMetadate("name", current);
			company = getMetadate("company", current);
			email = getMetadata("email", current);
			person = new Person();
			person.setCompany(company);
			person.setEmail(email);
			person.setName(name);
			persons[i] = person;
		}
		return persons;
	}
	
	/**
	 * 
	 * retrieve metadata from content, presentation, creation or update. 
	 * 
	 * @param s is the element name of root's sub element
	 * @param root is one of content, presentation, creation or update
	 * @return results as a String-Array
	 */
	private String[] getMetadata(String s, Element root) {
		if (root == null) return null;
		String[] metadata = null;
		List results;
		results = root.getChildren(s);
		if (results != null) {
			int size = results.size();
			metadata = new String[size];
			String metadate;
			int i;
			for (i = 0; i < size; i++) {
				metadate = ((Element) results.get(i)).getText();
				metadata[i] = metadate;
			}
		}
		return metadata;
	}
	
	private String getMetadate(String s, Element root) {
		if (root == null) return null;
		String metadate = null;
		Element current = root.getChild(s);
		if (current != null) {
			metadate = current.getText();
		}
		return metadate;
	}
	
	public Person[] getAuthors() {
		return authors;
	}
	public String getDate() {
		return date;
	}
	public String getDescription() {
		return description;
	}
	public Person[] getEditors() {
		return editors;
	}
	public String getSite() {
		return site;
	}
	public Person getSpeaker() {
		return speaker;
	}
	public String getTitle() {
		return title;
	}
}

