package eLecture;

import eLecture.annotation.Annotation;
import eLecture.annotation.Freehand;
import eLecture.annotation.Laserpointer;
import eLecture.annotation.Rectangle;
import eLecture.annotation.Textbox;
import generator.Constants;
import generator.GeneratorException;
import generator.SvgDocument;
import generator.SvgElement;
import generator.SVGUtil;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;


import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;

import sun.security.action.GetLongAction;

public class SvgGenerator {
	
	/**
	 * list of DocumentWrapper objects
	 */
	ArrayList outputDocs = null;
	/**
	 * overall number of slides
	 */
	private int slideCounter;
	private GeneratorHelper helper;
	private String separator;
	
	public SvgGenerator() {
		outputDocs = new ArrayList();
		separator = File.separator;
	}
	
	public static void main(String[] args) {
		SvgGenerator generator = new SvgGenerator();
		generator.proceed(args);
		System.out.println("file(s) generated!");
	}
	
	private void proceed(String[] args) {
		boolean iFlag = false;
		String inputFile = null;
		boolean dFlag = false;
		String inputDir = null;
		boolean lFlag = false;
		String lang = null;
		int i;
		int size = args.length;
		for (i = 0; i < size; i++) {
			if (args[i].equals("--help")) printHelp();
			// every flag can be specified at most once
			if (args[i].startsWith("-i:")) {
				if (iFlag) printInfo();
				iFlag = true;
				inputFile = getValue(args[i]);
				if (!inputFile.endsWith(".xml")) printInfo();
			}
			if (args[i].startsWith("-d:")) {
				if (dFlag) printInfo();
				dFlag = true;
				inputDir = getValue(args[i]);
			}
			if (args[i].startsWith("-l:")) {
				if (lFlag) printInfo();
				lFlag = true;
				lang = getValue(args[i]);
			}
		}
		// check for illegal states
		if (iFlag & dFlag) printInfo();
		if (!iFlag & !dFlag) printInfo();
		// proceed input
		try {
			if (lFlag) helper = new GeneratorHelper(lang);
			// use 'en' as default
			else helper = new GeneratorHelper("en"); 
		} catch (Exception e) {
			System.exit(0);
		}
		if (dFlag) {
			File d = new File(inputDir);
			if (!d.isDirectory()) printInfo();
			proceedDir(d);
		}
		else if(iFlag) {
			File input = new File(inputFile);
			if (!input.isFile()) printInfo();
			File output = new File(getOutput(inputFile));	
			proceedInput(input, output);
		}
	}
	
	private String getValue(String s) {
		s =  s.substring(3, s.length());
		return s;
	}
	
	private void printInfo() {
		System.err.println("Error: wrong usage! See '--help' " +
		"for information regarding usage.");
		System.exit(0);
	}
	
	private void printHelp() {
		System.out.println("Welcome to ICSY WebPresenter SVG Generator! This Java " +
				"programm proceeds XML files which are conform with the XML schema " +
				"icsy.xsd (and therefore describes lectures) and generates SVG " +
				"output files. This SVG output files are reconstructions of lectures" +
				" which are described through the XML input files. The output files " +
				"will be generated into the input directory and have the same name " +
				"as its input files (but the extension svg(z))");
		System.out.println("Usage info:");
		System.out.println("-i:<xml input file>");
		System.out.println("-d:<(nested) input file>");
		System.out.println("-l:<language id>");
		System.out.println("-i and -d cannot be specified at the same time");
		System.out.println("One out of -i and -d must be specified!"); 
		System.out.println("The output" +
				" file will be generated into the input directory.");
		System.out.println("if -d is specified, all generated files will be located " +
				" in the several input directory.");
		System.out.println("-l specifies the language used for the labels. The " +
				"specified value must match the name of the label sub element in " +
				"'labels.xml'. Default value is 'en'");
		System.out.println("All arguments which differs from above will be ignored.");
		System.exit(0);
	}
	
	/**
	 * 
	 * create new file name from <code>input</code> by changing suffix to "svg"
	 * 
	 * @param input file name
	 * @return new file name with a svg extension
	 */
	private String getOutput(String input) {
		String output = input.substring(0, input.lastIndexOf("."));
		output += ".svg";
		return output;
	}
	
	/**
	 * 
	 * proceed input dir. retrieve xml input files and construct output files. For 
	 * every input file create <code>DocumentWrapper</code>. 
	 * 
	 * @param inputDir input directory with input files. can be nested.
	 */
	private void proceedDir(File inputDir) {
		String[] fileNames = inputDir.list();
		int size = fileNames.length;
		int i; 
		File input, output;
		String name, path;
		// proceed all input files in dir
		for (i = 0; i < size; i++) {
			path = inputDir.getAbsolutePath();
			name = fileNames[i];
			input = new File(path, name);
			// check whether input is a file or dir
			// recursive call
			if (input.isDirectory()) {
				proceedDir(input);
			}
			else {
				if (name.endsWith(".xml")) {
					output = new File(path + separator + getOutput(name));
					proceedInput(input, output);
				}
			}
		}
	}
	
	/**
	 * 
	 * create <code>DocumentWrapper</code>-object
	 * 
	 * @param input XML input file
	 * @param output output file
	 */
	private void proceedInput(File input, File output) {
		try {
			System.out.println("Proceed input file: " + input.getCanonicalPath());
			DocumentWrapper doc = createDoc(input, output, false);
			// create another document which is scalable
        	   	String fileName = output.getAbsolutePath();
        	   	fileName = fileName.substring(0, fileName.lastIndexOf("."));
        		fileName += "_scale.svg";
			DocumentWrapper scalableDoc = createDoc(input, new File(fileName), true);
			build(doc);
			build(scalableDoc);
		} catch (FileNotFoundException e) {
			System.err.println("Error: File '" + input + "' not found!");
			System.exit(0);
		} catch (IOException e) {
			System.err.println("IO Exception occured while processing " +
					"file '" + input + "'");
			System.exit(0);
		} catch (JDOMException e) {
			System.err.println("Error occured while processing" +
					" '" + input + "'! Check xml markup of input file!");
			System.exit(0);
		}
	}
	
	/**
	 * 
	 * create DOM tree from input file and create <code>InputWrapper</code> and 
	 * <code>DocumentWrapper</code>
	 * 
	 * @param input XML input file
	 * @param output SVG output file
	 * @return <code>DocumentWrapper</code>-object
	 * @throws FileNotFoundException
	 * @throws IOException
	 * @throws JDOMException
	 */
	private DocumentWrapper createDoc(File input, File output, boolean scale) 
		throws FileNotFoundException, IOException, JDOMException {
		// validate against XML schema icsy.de
		SAXBuilder builder = 
            new SAXBuilder("org.apache.xerces.parsers.SAXParser", true);
        builder.setFeature("http://apache.org/xml/features/validation/schema", true);        
        builder.setProperty("http://apache.org/xml/properties/schema/" +
        		"external-schemaLocation", 
        		"http://www.icsy.de" +
        		" icsy.xsd");
           Document inputDoc = builder.build(new FileInputStream(input)); 
           InputWrapper inputWrapper = new InputWrapper(inputDoc); 
           DocumentWrapper docWrapper = new DocumentWrapper(inputWrapper, output);
           if (scale) docWrapper.setScale(true);
           return docWrapper;
	}
	
	/**
	 * for every input file create a <code>SvgDocument</code> object.
	 */
	private void build(DocumentWrapper docWrapper) {
		File output;
		SvgDocument doc;
		InputWrapper input = docWrapper.getInputWrapper();
		output = docWrapper.getOutput();
		doc = new SvgDocument(output);
		boolean scale = docWrapper.isScale();
		SvgElement root = initDoc(doc, input, scale); 
		accessExternalScript(root, "text/ecmascript", "RuntimeSetup.es");
		// copy script to dir of output file
		try {
			copy(new File("RuntimeSetup.es"), new File(output.getParent()));
		} catch (Exception e) {
			e.printStackTrace();
			// skip further process
			System.err.println("Error while generating '" + output.getName() + "'!"+
					" RuntimeSetup.es cannot be copied! Skip proceeding of file. " +
					"Continue with next input file!");
			return;
		}
		// define new context menu
		createContextMenu(root);
		// create group which consists of all topics; hide/unhide functionality
		SvgElement container = new SvgElement("g", root);
		root.addContent(container);
		container.addAttribute("id", "lecture");
		// proceed every topic
		Topic[] topics = input.getTopics();
		Topic topic;
		int numberTopics = topics.length;
		int i;
		// init slide progress bar panel
		// overall number of slides in file
		int nrSlides = getNrSlides(topics);
		long overallDur = input.getDuration();
		SvgElement progressBar = helper.initProgressPanel(root, nrSlides, 
				overallDur);
		SvgElement progressPanel = progressBar.getParent();
		Slide[] slides = getSlides(topics);
		SvgElement additionalInfoPanel = 
			helper.initAdditionalInfoPanel(progressPanel, slides);
		// for every file set counter to 0
		slideCounter = 0;
		
		for (i = 0; i < numberTopics; i++) {
			topic = topics[i];
			addSlides(container, topic.getSlides(), progressBar, 
					additionalInfoPanel, nrSlides);
		}
		docWrapper.setDoc(doc);
		store(docWrapper);
    }
	
	private void createContextMenu(SvgElement root) {
		SvgElement defs = SVGUtil.createElement("defs", root);
		Namespace ns = Namespace.getNamespace("custom", "http://customMenu.com");
		SvgElement menu = new SvgElement("menu", defs, ns);
		defs.addContent(menu);
		menu.addAttribute("id", "myCustomMenu");
		SvgElement e = SVGUtil.createElement("header", menu, "Custom Menu");
		e.setNS(ns);
		e = SVGUtil.createElement("item", menu, 
			new String[][] {{"action", "Quality"}}, 
			helper.getLabel("quality", false));
		e.setNS(ns);
		e = SVGUtil.createElement("item", menu, 
			new String[][] {{"action", "Pause"}, {"onactivate", "pause()"}},"Pause");
		e.setNS(ns);
		e = SVGUtil.createElement("item", menu, 
			new String[][] {{"action", "Mute"}}, helper.getLabel("mute", false));
		e.setNS(ns);
		e = SVGUtil.createElement("separator", menu);
		e.setNS(ns);
		
		e = SVGUtil.createElement("item", menu, 
			new String[][] {{"id", "item_infoPanel"}, 
			{"onactivate", "hideInfoPanel()"}, {"checked", "yes"}, 
			{"enabled", "no"}}, 
			helper.getLabel("hideInfoPanel", false));
			e.setNS(ns);
		
		e = SVGUtil.createElement("item", menu, 
			new String[][] {{"id", "item_timer"}, {"onactivate", "hideTimer()"}, 
			{"checked", "yes"}, {"enabled", "no"}}, helper.getLabel("hideTimer", 
				false));
			e.setNS(ns);	
			
		e = SVGUtil.createElement("separator", menu);
		e.setNS(ns);
		e = SVGUtil.createElement("item", menu, 
			new String[][] {{"action", "ViewSource"}}, 
			helper.getLabel("source", false));
		e.setNS(ns);
		e = SVGUtil.createElement("separator", menu);
		e.setNS(ns);
		e = SVGUtil.createElement("item", menu, 
			new String[][] {{"action", "Help"}}, 
			helper.getLabel("viewerHelp", false));
		e.setNS(ns);
		SvgElement howto = SVGUtil.createElement("item", menu, 
			new String[][] {{"target", "blank window"}}, 
			helper.getLabel("usage", false));
		howto.setNS(ns);
		Namespace xlink = Namespace.getNamespace("xlink", Constants.NS_XLINK);
		howto.addAttribute("href", Constants.WEBSITE_HELP, xlink);
		e = SVGUtil.createElement("separator", menu);
		e.setNS(ns);
		SvgElement provider = SVGUtil.createElement("item", menu, 
			new String[][] {{"target", "blank window"}}, 
			helper.getLabel("provider", false));
		provider.setNS(ns);
		provider.addAttribute("href", Constants.WEBSITE, xlink);
		e = SVGUtil.createElement("item", menu, 
			new String[][] {{"action", "About"}}, 	helper.getLabel("about", false));
		e.setNS(ns);
	}
	
	private void copy(File file, File targetFolder) throws Exception {
		// check whether file exists
		if (!file.isFile()) {
			// file does not exist
			return;
		}
		// create new file in target folder
		String folderName = targetFolder.getCanonicalPath();
		String fileName = file.getName();
		File copy = new File(folderName + separator + fileName);
		// check if file exists and if true skip
		if (copy.exists()) return;
		System.out.println("copy " + copy.getName() + " to " + folderName 
				+ separator);
		FileInputStream in = new FileInputStream(file); 
        FileOutputStream out = new FileOutputStream(copy); 
        int c; 
        while ((c = in.read()) != -1) {
           out.write(c); 
        }
        in.close(); 
        out.close(); 
	}
	
	private void accessExternalScript(SvgElement parent, String type, 
		String script) {
		SvgElement defs = SVGUtil.createElement("defs", parent);
		SvgElement child = new SvgElement("script", defs);
		defs.addContent(child);
		Namespace xlink = Namespace.getNamespace("xlink", Constants.NS_XLINK);
		Namespace a3 = Namespace.getNamespace("a3", Constants.NS_ADOBE_EXTENSIONS);
		child.addAttribute("href", script, xlink);	
		child.addAttribute("scriptImplementation", "Adobe", a3);
	}
	
	/**
	 * 
	 * create <code>SvgDocument</code> and add header to it. return root element 
	 * (svg)
	 * 
	 * @param doc <code>SvgDocument</code>
	 * @param input <code>InputWrapper</code>
	 * @return root element
	 */
	private SvgElement initDoc(SvgDocument doc, InputWrapper input, boolean scale) {
		SvgElement root = doc.getRoot();
		root.addAttribute("width", "100%");
		root.addAttribute("height", "100%");
		Slide[] slides = getSlides(input.getTopics());
		// if flag true then generate attribute 'viewBox' which causes auto scaling
		if (scale) {
			// resolution of first slide is default 
			root.addAttribute("viewBox", "0 0 " + slides[0].getWidth() + " " +
				slides[0].getHeight());
			// create set-attributes for svg-tag if slide resolutions differ
			updateViewBox(slides, root);
		}
		root.addAttribute("id", "doc");
		SvgElement title = SVGUtil.createElement("title", root);
		// set title of lecture
		Metadata metadata = input.getMetadata();
		String name = metadata.getTitle();
		title.addValue(name);
		String description = metadata.getDescription();
		if ((description != null) && (!description.trim().equals(""))) {
			SvgElement desc = SVGUtil.createElement("desc", root);
			desc.addValue(description);
		} 		
		return root;
	}
	
	private void updateViewBox(Slide[] slides, SvgElement svgTag) {
		// start with slide 2
		int size = slides.length;
		if (size > 1) {
			int i;
			Slide current, previous;
			int width, height, prevWidth, prevHeight;
			for (i = 1; i < size; i++) {
				current = slides[i];
				previous = slides[i - 1];
				width = current.getWidth();
				height = current.getHeight();
				prevWidth = previous.getWidth();
				prevHeight = previous.getHeight();
				// resolution differs from previous'
				if ((width != prevWidth) | (height != prevHeight)) {
					SVGUtil.createElement("set", svgTag, 
						new String[][] {{"attributeName", "viewBox"},
						{"begin", "slide" + (i - 1) + "_hide.end"},
						{"to", "0 0 " + width + " " + height}});
				}				
			}
		}
	}
	
	/**
	 * 
	 * calculate number of slides for a input file. number of slides is the sum of 
	 * all slides of every topic in an input file.
	 * 
	 * @param topics all topics of a file. at least one.
	 * @return overall number of slides
	 */
	private int getNrSlides(Topic[] topics) {
		int nrSlides = 0;
		int nrTopics = topics.length;
		int i;
		for (i = 0; i < nrTopics; i++) {
			nrSlides += topics[i].getSlides().length;
		}
		return nrSlides;
	}
	
	/**
	 * 
	 * get array of slides for all topics of a lecture. 
	 * 
	 * @param topics list of topics of a lecture
	 * @return list of slides
	 */
	private Slide[] getSlides(Topic[] topics) {
		Slide[] slides = null;
		// compute number of slides
		int nrSlides = getNrSlides(topics);
		// init array
		slides = new Slide[nrSlides];
		Slide[] currentSlides;
		Slide current;
		int k;
		int insertIndex = 0;
		int nrTopics = topics.length;
		int i;
		for (i = 0; i < nrTopics; i++) {
			currentSlides = topics[i].getSlides();
			for (k = 0; k < currentSlides.length; k++) {
				current = currentSlides[k];
				slides[insertIndex] = current;
				insertIndex++;
			}
		}
		return slides;
	}
	
	/**
	 * 
	 * for every slide generate markup and add it to <code>SvgDocument</code>. 
	 * <code>SvgElement</code> is the root element for the slides. 
	 * 
	 * @param root root element for the slides
	 * @param slides array of <code>Slide</code>-objects
	 * @param progressBar reference to root element of progress bar
	 * @param additionalInfoPanel reference to root element of additional info panel
	 * @param overallNrSlides number of slides of lecture
	 */
	private void addSlides(SvgElement root, Slide[] slides, SvgElement progressBar,
			SvgElement additionalInfoPanel, int overallNrSlides) {
		Slide slide;
		int size = slides.length;
        	int i;
        	SvgElement container, group;
        	// iterate all slides
        	for (i = 0; i < size; i++) {
        		slide = slides[i];
        		// every slide's components are grouped. group has unique id and is by
        		// default invisible
        		container = SVGUtil.createElement("g", root, 
        			new String[][] {{"id", "slide" + slideCounter}, {"opacity", "0"}});
        		// set point of start, duration of slide
        		long start = slide.getStart();
        		long dur = slide.getDuration();
        		long end = start + slide.getDuration();
        		// animation element makes slide visible
        		SvgElement setVisible = SVGUtil.createElement("set", container, 
        			new String[][] {{"attributeName", "opacity"}, {"to", "1"}});
        		// first slide. make slide visible after start button was clicked
        		if (slideCounter == 0) {
        			setVisible.addAttribute("begin", "start.click");
        		}
        		// make slide visible after last slide's hiding-animation was finished
        		else {
        			setVisible.addAttribute("begin", "slide" + (slideCounter -1) 
        					+ "_hide.end");
        		}
        		// hide slide after the end of the slide except last slide (freeze view)
        		if (slideCounter < (overallNrSlides - 1)) {
	        		SvgElement setHide = SVGUtil.createElement("animate", container,
	        			new String[][] {{"id", "slide" + slideCounter + "_hide"},
	        			{"attributeName", "opacity"}, {"to", "0"}, {"dur", "1s"},
	        			{"fill", "freeze"}});
	        		if (slideCounter == 0) {
	        			setHide.addAttribute("begin", setBegin(end + "ms"));
	        		}
	        		// calculate point of time for hiding slide (except last slide). 
	        		else {
	        			setHide.addAttribute("begin", "slide" + (slideCounter - 1) + 
	        					"_hide.end+" + dur + "ms");
	        		}
        		}
        		// add background image to slide
        		addImage(slide, container, slide.getWidth(), slide.getHeight());
        		// add ambient sound to slide
        		addSound(slide, container);
        		// make slide annotations hidable 
        		group = helper.createHidableGroup(container);
        		addAnnotations(slide, group);
        		// update progress panel to show metadata of slide
        		helper.updateProgressPanel(progressBar, additionalInfoPanel, slide, 
        				slideCounter, overallNrSlides);
        		slideCounter++;
        }
	}
	
	private void addImage(Slide slide, SvgElement root, int width, int height) {
		Namespace xlink = Namespace.getNamespace("xlink", Constants.NS_XLINK);
		SvgElement image = SVGUtil.createElement("image", root, 
			new String[][] {{"x", "0"}, {"y", "0"}, {"width", width + ""}, 
			{"height", height + ""}});
		image.addAttribute("href", slide.getImageUrl(), xlink);
	}
	
	private void addSound(Slide slide, SvgElement root) {
		String audio = slide.getAudioUrl();
		Namespace asv3 = Namespace.
			getNamespace("a", Constants.NS_ASV3);
		Namespace xlink = Namespace.getNamespace("xlink", Constants.NS_XLINK);
		SvgElement audioElement = new SvgElement("audio", root, asv3);
		audioElement.addAttribute("href", audio, xlink);
		audioElement.addAttribute("timeline", "independent", asv3);
		audioElement.addAttribute("begin", setBegin(slide.getStart() + "ms"));
		root.addContent(audioElement);
	}
	
	private void addAnnotations(Slide slide, SvgElement root) {
		Annotation[] annotations = slide.getAnnotations();
		// no annotations available, nothing to add
		if (annotations == null) return;
		Annotation annotation;
		int annotationSize = annotations.length;
		int i;
		String id;
		// iterate all annotations
		for (i = 0; i < annotationSize; i++) {
			annotation = annotations[i];
			id = "slide" + slideCounter + "l" + i;
			// check for kind of annotation
			if (annotation instanceof Laserpointer) {
				addLaserpointer(annotation, id, root);
			}
			else if (annotation instanceof Freehand) {
				addFreehandDrawing(annotation, id, root);
			}
			else if (annotation instanceof Rectangle) {
				addRectangle(annotation, id, root);
			}
			else {
				addTextbox(annotation, id, root);
			}
		}
	}
	
	/**
	 * 
	 * use the <code>Annotation</code>-object to create SVG makup for a laserpointer.
	 * 
	 * @param annotation provides information to build markup
	 * @param annotationId used to create an unique ID
	 * @param root attach laserpointer to root element
	 */
	private void addLaserpointer(Annotation annotation, String annotationId, 
		SvgElement root) {
		// cast into concrete Laserpointer object
		Laserpointer laserpointer = (Laserpointer) annotation;
		Location[] locations = laserpointer.getLocations();
		// if there are no locations exit method
		if ((locations == null) | (locations.length == 0)) {
			return;
		}
		// create laserpointer shape (circle)
		int r = laserpointer.getDiameter() / 2;
		// set default cx and cy values of circle (values of first location)
		SvgElement annotationElement = SVGUtil.createElement("circle", root,
			new String[][] {{"r", r + ""}, {"fill", annotation.getColor()},
			{"opacity", "0"}, {"cx", locations[0].getX() + ""},
			{"cy", locations[0].getY() + ""}});
		// point of start of animation
		long begin = annotation.getTimestamp();
		// create child elements for animation 
		// iterate all locations
		int locationSize = locations.length;
		SvgElement setX, setY;
		int x, y, lastX, lastY;
		Location location, nextLocation, lastLocation;
		long endTimestamp = 0;
		int k;
		long duration = 0;
		for (k = 0; k < locationSize - 1; k++) {
			location = locations[k];
			x = location.getX();
			y = location.getY();
			// flags. newX == false means that the current x-value == prev x-value
			boolean newX = true;
			boolean newY = true;
			if (k > 0) {
				lastLocation = locations[k - 1];
				lastX = lastLocation.getX();
				if (x == lastX) newX = false;
				lastY = lastLocation.getY();
				if (y == lastY) newY = false;
			}
			// retrieve duration
			if (newX | newY) {
				nextLocation = locations[k + 1];
				duration = nextLocation.getOffset();
			}
			
			// alter cx position of laserpointer
			if (newX) {
				setX = SVGUtil.createElement("set", annotationElement,
					new String[][] {{"begin", setBegin((begin + duration) + "ms")},
					{"attributeName", "cx"}, {"to", x + ""}});
			}
			// alter cy position of laserpointer
			if (newY) {
				setY = SVGUtil.createElement("set", annotationElement,
					new String[][] {{"begin", setBegin((begin + duration) + "ms")},
					{"attributeName", "cy"}, {"to", y + ""}});
			}
			// last location
			if (k == (locationSize - 2)) {
				endTimestamp = begin + duration;
			}
		}
		// set point of time for making the annotation visible
		SVGUtil.createElement("set", annotationElement,
			new String[][] {{"attributeName", "opacity"}, 
			{"begin", setBegin(begin + "ms")}, {"to", laserpointer.getOpacity()}});
		// at the end of last animation  set display to none
		SVGUtil.createElement("set", annotationElement,
			new String[][] {{"attributeName", "display"}, {"to", "none"},
			{"begin", setBegin(endTimestamp + "ms")}});
	}
	
	/*
	 * other possibility to realize laserpointer (maybe more overhead)
	 */
	private void addLaserpointerPath(Annotation annotation, String annotationId,
			SvgElement root) {
		Laserpointer laserpointer = (Laserpointer) annotation;
		// create laserpointer shape
		SvgElement annotationElement = new SvgElement("circle", root);
		root.addContent(annotationElement);
		annotationElement.addAttribute("id", annotationId);
		int r = laserpointer.getDiameter() / 2;
		annotationElement.addAttribute("r", r + "");
		annotationElement.addAttribute("fill", annotation.getColor());
		// start time of animation
		String begin = annotation.getTimestamp() + "ms";
		annotationElement.addAttribute("style", "fill-opacity:" + 
			annotation.getOpacity() + ";visibility:hidden;");
		// create child elements for animation 
		// iterate all locations and create animateMotion-Elements
		Location[] locations = annotation.getLocations();
		int locationSize = locations.length;
		SvgElement animateMotion;
		String animateMotionId;
		String path, x, y, nextX, nextY;
		Location location, nextLocation;
		int k;
		long duration;
		for (k = 0; k < locationSize - 1; k++) {
			location = locations[k];
			animateMotion = new SvgElement("animateMotion", annotationElement);
			annotationElement.addContent(animateMotion);
			animateMotionId = annotationId + "a" + k;
			animateMotion.addAttribute("id", animateMotionId);
			nextLocation = locations[k + 1];
			// first location element -> value of attribute begin is the 
			// timestamp of laserpointer
			if (k == 0) {						
				animateMotion.addAttribute("begin", setBegin(begin));
				duration = nextLocation.getOffset();
			}
			else {
				// start animation at the end of previous animation
				String predecessorId = annotationId + "a" + (k - 1);
				animateMotion.addAttribute("begin", predecessorId 
						+ ".end");
				duration = nextLocation.getOffset() - 	location.getOffset();
			}
			animateMotion.addAttribute("dur", duration + "ms");
			animateMotion.addAttribute("id", annotationId + "a" + k);
			// calculate path
			x = location.getX() + "";
			y = location.getY() + "";
			nextX = nextLocation.getX() + "";
			nextY = nextLocation.getY() + "";
			path = "M" + x + "," + y + " L" + nextX + "," + nextY;
			animateMotion.addAttribute("path", path);
		}
		// set point of time for making the annotation visible and invisible
		SvgElement setElement1 = new SvgElement("set", annotationElement);
		setElement1.addAttribute("attributeName", "visibility");
		setElement1.addAttribute("attributeType", "CSS");
		setElement1.addAttribute("begin", setBegin(begin));
		setElement1.addAttribute("to", "visible");
		SvgElement setElement2 = new SvgElement("set", annotationElement);
		setElement2.addAttribute("attributeName", "display");
		setElement2.addAttribute("attributeType", "CSS");
		// set display at the end of last animation to none
		String lastLocationId = annotationId + "a" + (locationSize - 2);
		setElement2.addAttribute("begin", lastLocationId + ".end");
		setElement2.addAttribute("to", "none");
		annotationElement.addContent(setElement1);
		annotationElement.addContent(setElement2);
	}
	
	
	
	/**
	 * 
	 * SVG element "line" is used to represent freehand drawing.
	 * A freehand drawing consists of one or more lines.
	 * At default x1=x2 and y1=y2. 
	 * "set" elements alter the x2 and y2 values to make the effect of a drawing.
	 * If height or width doesn't change then omit set element for x2 or y2.
	 * If duration of a line is >= 50 ms then use "animate" instead of "set".
	 * 
	 * @param annotation the <code>Annotation</code>-object which provides 
	 * information to build markup for freehand drawing
	 * @param annotationId used to build unique ID
	 * @param root attach freehand drawing to root element
	 */
	private void addFreehandDrawing(Annotation annotation, String annotationId,
			SvgElement root) {
		Freehand freehand = (Freehand) annotation;
		String color = freehand.getColor();
		int diameter = freehand.getDiameter();
		String opacity = freehand.getOpacity();
		long timestamp = freehand.getTimestamp();
		Location[] locations = freehand.getLocations();
		// iterate all locations (except the last one) and create for each location 
		// a line element and 2 animate elements for x2 and y2
		int i;
		int size = locations.length;
		long begin, duration;
		Location location, nextLocation;
		SvgElement line;
		for (i = 0; i < (size - 1); i++) {
			location = locations[i];
			nextLocation = locations[i + 1];
			line = SVGUtil.createElement("line", root, 
				new String[][] {{"stroke", color},
				{"stroke-width", diameter + ""}, {"opacity", opacity}, 
				{"x1", location.getX() + ""}, {"y1", location.getY() + ""},
				{"x2", location.getX() + ""}, {"y2", location.getY() + ""}});
			// add child elements
			begin = timestamp + location.getOffset();
			duration = nextLocation.getOffset() - location.getOffset();
			int from = location.getX();
			int to = nextLocation.getX();
			if (from != to) {
				if (duration < 50) {
					SVGUtil.createElement("set", line,
						new String[][] {{"begin", setBegin((begin + duration) + "ms")},
						{"attributeName", "x2"}, {"to", to + ""}});
				}
				else {
					SVGUtil.createElement("animate", line,
						new String[][] {{"begin", setBegin(begin + "ms")},
						{"dur", duration + "ms"}, {"to", to + ""}, 
						{"attributeName", "x2"}, {"fill", "freeze"}});
				}
			}
			from = location.getY();
			to = nextLocation.getY();
			if (from != to) {
				if (duration < 50) {
					SVGUtil.createElement("set", line, 
					new String[][] {{"begin", setBegin((begin + duration) + "ms")},
						{"attributeName", "y2"}, {"to", to + ""}});
				}
				else {
					SVGUtil.createElement("animate", line, 
						new String[][] {{"begin", setBegin(begin + "ms")},
						{"dur", duration + "ms"}, {"fill", "freeze"},
						{"attributeName", "y2"}, {"to", to + ""}});
				}
				
			}
		}
	}
	
	/**
	 * 
	 *  Algorithm:
	 * (create rect element)
	 * 1. retrieve x and y value of annotation
	 * 2. create rect: <svg:rect x="<x of annotation>" y="<y of annotation>" 
	 *                 width="0" height="0" ... >
	 * 3. retrieve timestamp (point of time the annotation starts)                
	 * (for every annotation location (except the last): create set elements and add 
	 * it to rect)
	 * 4. retrieve offsetX1 (offsetX of current location) from annotation
	 * 5. retrieve offsetX2 (offsetX of next location) from annotation
	 * 6. width += offsetX2 - offsetX1;
	 * 7. duration = nextLocation.getOffset() - location.getOffset();
	 * 8. calculate value for attribute begin of set element:
	 *    begin = timestamp + location.getOffset() + duration;
	 * 9. use variables x_ and width_ for attr x and attr width of set element:
	 *    9.1 if width >= 0 then 
	 *    	  width_ = width
	 *        create set element for width: 
	 *        <svg:set attributeName="width" begin="<begin>" to="<width_>" ...
	 *    9.2 if width < 0 then
	 *    	  x_ = x + width
	 *        width_ = |width|
	 *        create set element for width:
	 *        <svg:set attributeName="width" begin="<begin>" to="<width_>" ...
	 *        create set element for x:
	 *        <svg:set attributeName="x" begin="<begin>" to="<x_>" ...
	 * 10. do the same for height and y
	 * 
	 * 
	 * Optimization: if from = to then omit generation of set element
	 * 
	 * @param annotation <code>Annotation</code>-object provides information to 
	 * create markup for textmarker
	 * @param annotationId used to construct unique ID
	 * @param root attach textmarker to root element
	 */
	private void addRectangle(Annotation annotation, String annotationId, 
			SvgElement root) {
		Rectangle rectangle = (Rectangle) annotation;
		// color of rect element
		String color = annotation.getColor();
		// point of start time of annotation
		long timestamp = annotation.getTimestamp();
		// opacity of rect element
		String opacity = annotation.getOpacity();
		// create rect element
		int x = rectangle.getX();
		int y = rectangle.getY();
		// helper variables for set elements
		int x_ = x;
		int y_ = y;
		// by default width and height are 0 
		int width = 0;
		int height = 0;
		int width_ = width;
		int height_ = height;
		SvgElement rectElement = SVGUtil.createElement("rect", root,
			new String[][] {{"x", x + ""}, {"y", y + ""}, {"width", width + ""},
			{"height", height + ""}, {"fill", color}, {"opacity", opacity}});
		// iterate all locations of the annotation
		Location[] locations = annotation.getLocations();
		Location location, nextLocation;
		int size = locations.length;
		int i;
		// begin attribute of set element
		long begin;
		// duration is needed to calculate begin
		long duration;
		// set elements for width, height, x and y, respectively
		SvgElement setWidth, setHeight, setX, setY;
		// offsetX of current location, offsetY of current location, offsetX of
		// next location and offsetY of next location, respectively
		int offsetX1, offsetY1, offsetX2, offsetY2;
		for (i = 0; i < (size - 1); i++) {
			location = locations[i];
			nextLocation = locations[i + 1];
			begin = timestamp + location.getOffset();
			offsetX1 = location.getOffsetx();
			offsetX2 = nextLocation.getOffsetx();
			duration = nextLocation.getOffset() - location.getOffset();
			boolean width_flag = true;
			// calculate new width
			width += offsetX2 - offsetX1;
			if (width == 0) width_flag = false;
			boolean alterX = false;
			if (width < 0) {
				x_ = x + width;
				width_ = (-1) * width;
				alterX = true;
			}
			else {
				width_ = width;
			}
			if (width_flag) {
				if (alterX) {
					setX = new SvgElement("set", rectElement);
					setX.addAttribute("attributeName", "x");
					setX.addAttribute("begin", setBegin((begin + duration) + "ms"));
					setX.addAttribute("to", x_ + "");
					rectElement.addContent(setX);
				}
				setWidth = new SvgElement("set", rectElement);
				setWidth.addAttribute("attributeName", "width");
				setWidth.addAttribute("begin", setBegin((begin + duration) + "ms"));
				setWidth.addAttribute("to", width_ + "");
				rectElement.addContent(setWidth);
				
			}
			offsetY1 = location.getOffsety();
			offsetY2 = nextLocation.getOffsety();
			height += offsetY2 - offsetY1;
			boolean height_flag = true;
			if (height == 0) height_flag = false;
			boolean alterY = false;
			if (height < 0) {
				y_ = y + height;
				height_ = (-1) * height;
				alterY = true;
			}
			else {
				height_ = height;
			}
			if (height_flag) {
				if (alterY) {
					setY = new SvgElement("set", rectElement);
					rectElement.addContent(setY);
					setY.addAttribute("attributeName", "y");
					setY.addAttribute("begin", setBegin((begin + duration) + "ms"));
					setY.addAttribute("to", y_ + "");
				}
				setHeight = new SvgElement("set", rectElement);
				setHeight.addAttribute("attributeName", "height");
				setHeight.addAttribute("begin", setBegin((begin + duration) 
						+ "ms"));
				setHeight.addAttribute("to", height_ + "");
				rectElement.addContent(setHeight);
			}
			
		}
	}
	
	
	private void addTextbox(Annotation annotation, String annotationId, 
			SvgElement root) {
		Textbox textbox = (Textbox) annotation;
		SvgElement group = SVGUtil.createElement("g", root,
			new String[][] {{"id", annotationId}, {"opacity", "0"}});
		String opacity = textbox.getOpacity();
		SVGUtil.createElement("animate", group, 
			new String[][] {{"attributeName", "opacity"}, {"from", "0"}, 
			{"to", opacity}, {"begin", setBegin(textbox.getTimestamp() + "ms")}, 
			{"dur", "1s"}, {"fill", "freeze"}});
		// create rounded textbox
		// minimal width is 110 px
		int width = textbox.getWidth();
		if (width < 110) width = 110;
		int height = textbox.getHeight();
		SvgElement rect = SVGUtil.createElement("rect", group,
			new String[][] {{"rx", "15"}, {"ry", "15"}, {"width", width + ""},
			{"height", height + ""}});
		String fillType = textbox.getFillType();
		if (fillType.equals("filled") | fillType.equals("framed_filled")) {
			rect.addAttribute("fill", textbox.getColor());
		}
		else rect.addAttribute("fill", "none");
		if (fillType.equals("framed") | fillType.equals("framed_filled")) {
			rect.addAttribute("stroke", textbox.getColor());
			rect.addAttribute("stroke-width", "1");
		}
		
		// x and y value depends on position of bubble connection
		// bubble connection
		SvgElement connection = SVGUtil.createElement("polygon", group);
		/*
		 * There are different conditions which define the position:
		 * 1) connectY >= y -> connection is on top of bubble 
		 *    connectY < y -> connection is on bottom of bubble
		 * 2) x <= connectX -> peak of connection points to left (position on 
		 * 	  left side of bubble)
		 *    x > connectX -> peak points to right (position on right side)
		 */
		if (fillType.equals("filled") | fillType.equals("framed_filled")) {
			connection.addAttribute("fill", textbox.getColor());
		}
		else connection.addAttribute("fill", "none");
		if (fillType.equals("framed") | fillType.equals("framed_filled")) {
			connection.addAttribute("stroke", textbox.getColor());
			connection.addAttribute("stroke-width", "1");
		}
		int connectX = textbox.getConnectX();
		int connectY = textbox.getConnectY();
		int x = textbox.getX();
		int y = textbox.getY();
		String point1, point2, point3;
		int rect_x, rect_y;
		if (connectY >= y) {
			// connection is on top of bubble
			rect_x = textbox.getConnectX();
			rect_y = textbox.getConnectY();
			rect.addAttribute("x", rect_x + "");
			rect.addAttribute("y", rect_y + "");
			if (x <= connectX) {
				// rx, ry = 25
				point1 = (connectX + 25) + "," + connectY;
				// connection line between bubble and bubble-connection is 50 px
				point2 = (connectX + 25 + 50) + "," + connectY;
				point3 = x + "," + y;
			}
			else {
				point1 = ((connectX + width) - 25) + "," + connectY;
				point2 = ((connectX + width) - 25 - 50) + "," + connectY;
				point3 = x + "," + y;
			}
		}
		else {
			// connection is on bottom of bubble
			rect_x = textbox.getConnectX();
			rect_y = textbox.getConnectY() - height;
			rect.addAttribute("x", rect_x + "");
			rect.addAttribute("y", rect_y +  "");
			if (x <= connectX) {
				point1 = (connectX + 25) + "," + connectY;
				point2 = (connectX + 25 + 50) + "," + connectY;
				point3 = x + "," + y;
			}
			else {
				point1 = ((connectX + width) - 25) + "," + connectY;
				point2 = ((connectX + width) -25 - 50) + "," + connectY;
				point3 = x + "," + y;
			}
		}
		String points = point1 + " " + point2 + " " + point3;
		connection.addAttribute("points", points);
		addText(group, textbox, rect_x, rect_y);
	}
	
	/**
	 * 
	 * add text elements to textbox. For every line of the textbox create a text 
	 * element.
	 * 
	 * @param root add text element to root element
	 * @param textbox <code>Textbox</code>-object
	 * @param rect_x x-coordinate of textbox
	 * @param rect_y y-coordinate of textbox
	 */
	private void addText(SvgElement root, Textbox textbox, int rect_x, int rect_y) {
		String[] lines = textbox.getLines();
		int size = lines.length;
		int i;
		// for every line generate a text element
		SvgElement text;
		String line;
		int fontSize = (int) textbox.getFontSize();
		// y value of first text element
		int y = rect_y + (2 * fontSize);
		for (i = 0; i < size; i++) {
			line = lines[i];
			// add sans-serif as alternative font-family
			text = SVGUtil.createElement("text", root, new String[][] {{"y", y + ""},
				{"x", (rect_x + fontSize) + ""}, {"text-anchor", "start"},
				{"font-size", fontSize + ""}, 
				{"font-family", textbox.getPreferredFont() + ", sans-serif"},
				{"fill", textbox.getTextColor()}});
			// y value of next text element
			y += 1.2 * fontSize;
			String style = textbox.getFontStyle();
			String fontWeight = "normal";
			if (style.equals("bold")) fontWeight = "bold";
			text.addAttribute("font-weight", fontWeight);
			if (style.equals("bold_italic") | style.equals("italic")) {
				text.addAttribute("font-style", "italic");
			}
			text.addText(line);
		}
	}
	
	/**
	 * 
	 * return value for attribute begin of a SVG animation element. because 
	 * animation doesn't start when document is loaded a relative value for the 
	 * attribute begin of an animation element is necessary. For the first slide the
	 * point of reference is the event start.click (click on the start button). For 
	 * further slides the point of reference is "slide_<int-value>_hide.end".
	 * 
	 * @param timestamp in millis
	 * @return value for attribute begin
	 */
	private String setBegin(String timestamp) {
		if (slideCounter == 0)
		return "start.click+" + timestamp;
		else {
			return "slide" + (slideCounter - 1) + "_hide.end+" + timestamp;
		}
	}
	
	/**
	 * for every <code>DocumentWrapper</code>-object store SVG document to output 
	 * file.
	 */
	private void store(DocumentWrapper docWrapper) {
		SvgDocument doc;
		doc = docWrapper.getDoc();
		try {
			doc.store();
		} catch (GeneratorException e) {
			System.err.println("Error: '" + doc.getOutputFile().getName()
					+ "' couldn't be created!\n");
		}
	}
}
