package eLecture.annotation;

import eLecture.Location;

/**
 * @author seb
 *
 * represent an annotation. provides information to construct SVG markup.
 * 
 */
public abstract class Annotation {
	/**
	 * point of start in millis
	 */
	long timestamp;
	String color;
	String transparency;
	/**
	 * list of <code>Location</code>-objects
	 */
	Location[] locations;
	
	public Annotation(long timestamp, String color, String transparency, 
			Location[] locations) {
		this.timestamp = timestamp;
		this.color = color;
		this.transparency = transparency;
		this.locations = locations;
	}

	public String getColor() {
		return color;
	}

	public Location[] getLocations() {
		return locations;
	}

	public long getTimestamp() {
		return timestamp;
	}

	public String getTransparency() {
		return transparency;	
	}
	
	/**
	 * 
	 * From transparency calculate opacity. Opacity is the opposite of transparency.
	 * 
	 * @return opacity
	 */
	public String getOpacity() {
		double temp, opacity;
		try {
			temp = Double.parseDouble(transparency);
			opacity = 1.0 - temp;
		} catch (NumberFormatException e) {
			opacity = 1;
		}
		return opacity + "";
	}
}
