/*
 * Created on 14.07.2005
 *
 */
package generator;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.zip.GZIPOutputStream;

import org.jdom.DocType;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Namespace;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * @author seb
 * 
 * class maintain DOM tree of lecture. provides methods to store tree into a file and 
 * compress SVG file.
 */
/**
 * @author seb
 *
 */
public class SvgDocument {
    
    private Document doc = null;
    private SvgElement svgRoot;
    private File outputFile = null;
    
    public SvgDocument() {
    		this(null);
    }
    
    public SvgDocument(File outputFile) {
        this.outputFile = outputFile;
    		Element root = 
            new Element("svg");
        svgRoot = new SvgElement(root);   
        svgRoot.setNS(Constants.NS_SVG);        
        svgRoot.addAttribute("version", "1.2");
        Namespace a3 = Namespace.getNamespace("a3", Constants.NS_ADOBE_EXTENSIONS);
        svgRoot.addAttribute("scriptImplementation", "Adobe", a3);
        svgRoot.addAttribute("zoomAndPan", "disable");
        // on document load convert text:wrap tags into regular svg text tags
        // init user provided context menu
        svgRoot.addAttribute("onload", "init(evt)");
        DocType docType = new DocType("svg", "-//W3C//DTD SVG 1.0//EN", 
                "http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd");
        doc = new Document(root, docType);
    }
    
    public SvgElement getRoot() {
        return svgRoot;
    }
    
    public Document getDocument() {
        return doc;
    }   
           
    /**
     * 
     * store DOM tree to output file. use iso-8859-1 encoding
     * 
     * @throws GeneratorException
     */
    public void store() throws GeneratorException {
        Format format = Format.getPrettyFormat();
        format.setEncoding("iso-8859-1");
        // use tabs as indent
        format.setIndent("\t");
        XMLOutputter outputter = new XMLOutputter(format);
        try { 
            outputter.output(doc, new FileOutputStream(outputFile));
            // create compressed svg file
            compressFile(outputFile);
            } catch (FileNotFoundException e) {
            System.err.println("fileNotFoundException");
            throw new GeneratorException();
        } catch (IOException e) {
            System.err.println("IoException");
            throw new GeneratorException();
        }  
    }
    
    /**
     * 
     * create GZIP archiev from <code>svgFile</code>
     * 
     * @param svgFile file to compress
     */
    private void compressFile(File svgFile) {
    		try {
    			// Create the GZIP output stream - attach "z" on filename .(svgz)
    			String output = svgFile + "z";
    			GZIPOutputStream out = 
    				new GZIPOutputStream(new FileOutputStream(output));
    			// open the input file
    	        	FileInputStream in = new FileInputStream(svgFile);
    	        	// transfer bytes from the input file to the GZIP output stream
    	        	byte[] buffer = new byte[1024];
    	        	int length;
    	        	while ((length = in.read(buffer)) > 0) {
    	        		out.write(buffer, 0, length);
    	        	}
    	        	in.close();
    	        	// complete the GZIP file
    	        	out.finish();
    	        	out.close();
    		} catch (IOException e) {
    	    		System.err.println("Error: Couldn't compress " + svgFile.getName() + 
    	    				" to " + svgFile.getName() + "z");
    	    		System.exit(0);
    	    }
    }
    
    /**
     * 
     * print document to stdout
     * 
     * @param doc svg document
     */
    /**
     * @param doc
     */
    public void print(Document doc) {
        Format format = Format.getPrettyFormat();
        format.setEncoding("iso-8859-1");
        format.setIndent("\t");
        XMLOutputter outputter = new XMLOutputter(format);
        try {
            outputter.output(doc, System.out);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

	public File getOutputFile() {
		return outputFile;
	}    
	
	public void setOutputFile(File file) {
		outputFile = file;
	}
}
