/*
 * Created on 14.07.2005
 *
 */
package generator;

import org.jdom.Attribute;
import org.jdom.CDATA;
import org.jdom.Element;
import org.jdom.Namespace;

/**
 * @author seb
 *
 * an abstraction of a JDOM element. provides methods to add attributes, sub elements 
 * and text content.
 *
 */
/**
 * @author seb
 *
 */
/**
 * @author seb
 *
 */
public class SvgElement {
    
    /**
     * element name
     */
    private String name;
    /**
     * reference to element
     */
    private Element element;
    /**
     * reference to root element
     */
    private Element parent = null;
    
    SvgElement(Element element) {
        this.name = element.getName();
        this.element = element;
    }
    
    public SvgElement(String name) {
        element = new Element(name);
        this.name = name;
    }
    
   /**
    * 
    * create element with the default namespace SVG
    * 
    * @param name element name
    * @param parent root element
    */
    public SvgElement(String name, SvgElement parent) {
    		this(name, parent, Namespace.getNamespace("svg", Constants.NS_SVG));
    }
   
   /**
    * 
    * create element with given Namespace
    * 
    * @param name
    * @param parent
    * @param ns
    */
    public SvgElement(String name, SvgElement parent, Namespace ns) {
       this.name = name;
       element = new Element(name, ns);
       this.parent = parent.getElement();
    }
      
    /**
     * 
     * add attribute with given value to element
     * 
     * @param name attribute name
     * @param value attribute value
     */
    public void addAttribute(String name, String value) {
        Attribute attr = new Attribute(name, value);
        element.setAttribute(attr);
    }   
    
    /**
     * 
     * add attribute with given value and namespace to element
     * 
     * @param name attribute name
     * @param value attribute value
     * @param ns namespace
     */
    public void addAttribute(String name, String value, Namespace ns) {
    		Attribute attr = new Attribute(name, value, ns);
    		element.setAttribute(attr);
    }
    
    /**
     * 
     * add sub element to element
     * 
     * @param child new sub element
     */
    public void addContent(SvgElement child) {
        Element c = child.getElement();
        element.addContent(c);
    }
    
    public void addText(CDATA content) {
    		element.addContent(content);
    }
    
    /**
     * 
     * add text to element
     * 
     * @param content
     */
    public void addText(String content) {
    		element.addContent(content);
    }
    
    public void addValue(String value) {
        element.setText(value);
    }
    
    public void setNS(String ns) {
        Namespace namespace = Namespace.getNamespace(Constants.NS_SVG);
        element.setNamespace(namespace);
    }
    
    public void setNS(Namespace ns) {
    		element.setNamespace(ns);
    }
       
    public Element getElement() {
        return element;
    }

	public SvgElement getParent() {
		return new SvgElement(parent);
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}       
}
